@@ -139,6 +139,7 @@ static constexpr const input_function_metadata_t input_function_metadata[] = {
     {L"insert-line-over", readline_cmd_t::insert_line_over},
     {L"insert-line-under", readline_cmd_t::insert_line_under},
     {L"kill-bigword", readline_cmd_t::kill_bigword},
+    {L"kill-inner-line", readline_cmd_t::kill_inner_line},
     {L"kill-line", readline_cmd_t::kill_line},
     {L"kill-selection", readline_cmd_t::kill_selection},
     {L"kill-whole-line", readline_cmd_t::kill_whole_line},
@@ -351,6 +352,10 @@ void inputter_t::select_interrupted() /* override */ {
     this->push_front(char_event_t{char_event_type_t::check_exit});
 }
 
+void inputter_t::uvar_change_notified() /* override */ {
+    this->parser_->sync_uvars_and_fire(true /* always */);
+}
+
 void inputter_t::function_push_arg(wchar_t arg) { input_function_args_.push_back(arg); }
 
 wchar_t inputter_t::function_pop_arg() {
