@@ -137,7 +137,7 @@ int execute_setpgid(pid_t pid, pid_t pgroup, bool is_parent) {
     }
 }
 
-int child_setup_process(pid_t new_termowner, pid_t fish_pgrp, const job_t &job, bool is_forked,
+int child_setup_process(pid_t claim_tty_from, const job_t &job, bool is_forked,
                         const dup2_list_t &dup2s) {
     // Note we are called in a forked child.
     for (const auto &act : dup2s.get_actions()) {
@@ -161,7 +161,7 @@ int child_setup_process(pid_t new_termowner, pid_t fish_pgrp, const job_t &job,
             return err;
         }
     }
-    if (new_termowner != INVALID_PID && new_termowner != fish_pgrp) {
+    if (claim_tty_from >= 0 && tcgetpgrp(STDIN_FILENO) == claim_tty_from) {
         // Assign the terminal within the child to avoid the well-known race between tcsetgrp() in
         // the parent and the child executing. We are not interested in error handling here, except
         // we try to avoid this for non-terminals; in particular pipelines often make non-terminal
@@ -170,12 +170,10 @@ int child_setup_process(pid_t new_termowner, pid_t fish_pgrp, const job_t &job,
         // another process which may happen if we are run in the background with job control
         // enabled. Note if stdin is not a tty, then tcgetpgrp() will return -1 and we will not
         // enter this.
-        if (tcgetpgrp(STDIN_FILENO) == fish_pgrp) {
-            // Ensure this doesn't send us to the background (see #5963)
-            signal(SIGTTIN, SIG_IGN);
-            signal(SIGTTOU, SIG_IGN);
-            (void)tcsetpgrp(STDIN_FILENO, new_termowner);
-        }
+        // Ensure this doesn't send us to the background (see #5963)
+        signal(SIGTTIN, SIG_IGN);
+        signal(SIGTTOU, SIG_IGN);
+        (void)tcsetpgrp(STDIN_FILENO, getpid());
     }
     sigset_t sigmask;
     sigemptyset(&sigmask);
@@ -283,12 +281,9 @@ posix_spawner_t::posix_spawner_t(const job_t *j, const dup2_list_t &dup2s) {
     // desired_pgid tracks the pgroup for the process. If it is none, the pgroup is left unchanged.
     // If it is zero, create a new pgroup from the pid. If it is >0, join that pgroup.
     maybe_t<pid_t> desired_pgid = none();
-    if (auto job_pgid = j->group->get_pgid()) {
-        desired_pgid = *job_pgid;
-    } else {
-        assert(j->group->needs_pgid_assignment() && "We should be expecting a pgid");
-        // We are the first external proc in the job group. Set the desired_pgid to 0 to indicate we
-        // should creating a new process group.
+    if (auto pgid = j->group->get_pgid()) {
+        desired_pgid = *pgid;
+    } else if (j->processes.front()->leads_pgrp) {
         desired_pgid = 0;
     }
 
@@ -374,44 +369,57 @@ void safe_report_exec_error(int err, const char *actual_cmd, const char *const *
                             const char *const *envv) {
     switch (err) {
         case E2BIG: {
-            char sz1[128], sz2[128];
+            char sz1[128];
 
             long arg_max = -1;
 
             size_t sz = 0;
+            size_t szenv = 0;
             const char *const *p;
             for (p = argv; *p; p++) {
                 sz += std::strlen(*p) + 1;
             }
 
             for (p = envv; *p; p++) {
-                sz += std::strlen(*p) + 1;
+                szenv += std::strlen(*p) + 1;
             }
+            sz += szenv;
 
             format_size_safe(sz1, sz);
             arg_max = sysconf(_SC_ARG_MAX);
 
             if (arg_max > 0) {
                 if (sz >= static_cast<unsigned long long>(arg_max)) {
+                    char sz2[128];
                     format_size_safe(sz2, static_cast<unsigned long long>(arg_max));
-                    FLOGF_SAFE(exec,
-                               "Failed to execute process '%s': the size of argument and "
-                               "environment lists %s exceeds the OS limit of %s.",
-                               actual_cmd, sz1, sz2);
+                    FLOGF_SAFE(
+                        exec,
+                        "Failed to execute process '%s': the total size of the argument list and "
+                        "exported variables (%s) exceeds the OS limit of %s.",
+                        actual_cmd, sz1, sz2);
                 } else {
                     // MAX_ARG_STRLEN, a linux thing that limits the size of one argument. It's
                     // defined in binfmts.h, but we don't want to include that just to be able to
                     // print the real limit.
                     FLOGF_SAFE(exec,
-                               "Failed to execute process '%s': An argument exceeds the OS "
-                               "argument length limit.");
+                               "Failed to execute process '%s': An argument or exported variable "
+                               "exceeds the OS "
+                               "argument length limit.",
+                               actual_cmd);
+                }
+
+                if (szenv >= static_cast<unsigned long long>(arg_max) / 2) {
+                    FLOGF_SAFE(exec,
+                               "Hint: Your exported variables take up over half the limit. Try "
+                               "erasing or unexporting variables.");
                 }
             } else {
-                FLOGF_SAFE(exec,
-                           "Failed to execute process '%s': The total size of the argument and "
-                           "environment lists (%s) exceeds the "
-                           "operating system limit.",
-                           actual_cmd, sz1);
+                FLOGF_SAFE(
+                    exec,
+                    "Failed to execute process '%s': the total size of the argument list and "
+                    "exported variables (%s) exceeds the "
+                    "operating system limit.",
+                    actual_cmd, sz1);
             }
             break;
         }
@@ -465,6 +473,11 @@ void safe_report_exec_error(int err, const char *actual_cmd, const char *const *
                                "executable command.",
                                actual_cmd, interpreter);
                 }
+            } else if (access(actual_cmd, X_OK) == 0) {
+                FLOGF_SAFE(exec,
+                           "Failed to execute process '%s': The file exists and is executable. "
+                           "Check the interpreter or linker?",
+                           actual_cmd);
             } else {
                 FLOGF_SAFE(exec,
                            "Failed to execute process '%s': The file does not exist or could not "
