@@ -132,7 +132,10 @@ long fish_wcstol(const wchar_t *str, const wchar_t **endptr = nullptr, int base
 long long fish_wcstoll(const wchar_t *str, const wchar_t **endptr = nullptr, int base = 10);
 unsigned long long fish_wcstoull(const wchar_t *str, const wchar_t **endptr = nullptr,
                                  int base = 10);
+double fish_wcstod(const wchar_t *str, wchar_t **endptr, size_t len);
 double fish_wcstod(const wchar_t *str, wchar_t **endptr);
+double fish_wcstod(const wcstring &str, wchar_t **endptr);
+double fish_wcstod_underscores(const wchar_t *str, wchar_t **endptr);
 
 /// Class for representing a file's inode. We use this to detect and avoid symlink loops, among
 /// other things. While an inode / dev pair is sufficient to distinguish co-existing files, Linux
